import { GameRule, system, world } from "@minecraft/server";
import { afterEvents, beforeEvents, sleep, Vector } from "./libraries/utils";
afterEvents.playerPlaceBlock.subscribe(({ block, dimension, player }) => {
    if (block.typeId !== 'minecraft:carved_pumpkin')
        return;
    const origin = block.below();
    if (origin?.typeId !== 'minecraft:hay_block')
        return;
    origin.destroy(false);
    block.destroy(false);
    const golem = dimension.spawnEntity('rhg:hay_golem', origin.bottomCenter());
    golem.teleport(golem.location, { facingLocation: player.getHeadLocation() });
    const tameable = golem.getComponent('tameable');
    tameable.tame(player);
});
beforeEvents.itemUseOn.subscribe(async ({ block, itemStack: item, source: player }) => {
    if (block.typeId !== 'minecraft:pumpkin' || item.typeId !== 'minecraft:shears')
        return;
    await sleep(0);
    const origin = block.below();
    if (origin?.typeId !== 'minecraft:hay_block')
        return;
    origin.destroy(false);
    block.destroy(false);
    const golem = block.dimension.spawnEntity('rhg:hay_golem', origin.bottomCenter());
    golem.teleport(golem.location, { facingLocation: player.getHeadLocation() });
    const tameable = golem.getComponent('tameable');
    tameable.tame(player);
});
const cooldown = new Map();
system.afterEvents.scriptEventReceive.subscribe(({ id, sourceEntity: golem }) => {
    if (golem?.typeId !== 'rhg:hay_golem')
        return;
    switch (id.split(':')[1]) {
        case 'item': {
            if (!cooldown.has(golem.id))
                DisplayItem(golem);
            break;
        }
        case 'chest': {
            Chest(golem);
            break;
        }
        default:
            break;
    }
}, { namespaces: ['rhg'] });
function DisplayItem(golem) {
    cooldown.set(golem.id, system.runTimeout(() => cooldown.delete(golem.id), 10));
    const inv = golem.inventory?.container;
    if (!inv)
        return;
    for (let i = 0; i < inv.size; i++) {
        const item = inv.getItem(i);
        if (!item)
            continue;
        golem.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 ${item.typeId}`);
        return;
    }
    golem.runCommand(`replaceitem entity @s slot.weapon.mainhand 0 air`);
}
async function Chest(golem) {
    const { dimension, location } = golem;
    let block = golem.getBlockFromViewDirection({ maxDistance: 2, includeLiquidBlocks: false, includePassableBlocks: false })?.block;
    let container = block?.inventory?.container;
    if (!container || container.size < 26) {
        for (const direction of [Vector.left, Vector.right, Vector.back, Vector.forward, Vector.down]) {
            block = dimension.getBlock(Vector.add(location, direction));
            container = block?.inventory?.container;
            if (container && container.size >= 26)
                break;
        }
        if (!container || container.size < 26)
            return;
        golem.setRotation(Vector.toVector2(Vector.subtract(block, location)));
    }
    const isBarrel = block?.typeId === 'minecraft:barrel';
    dimension.playSound(isBarrel ? 'block.barrel.open' : 'random.chestopen', block.center());
    await sleep(20);
    const inv = golem.inventory?.container;
    for (let i = 0; i < inv.size; i++) {
        const item = inv.getItem(i);
        if (!item || item.typeId === 'minecraft:bone_meal')
            continue;
        const left = container.addItem(item);
        if (!left) {
            inv.setItem(i);
            continue;
        }
        else if (left.amount === item.amount)
            continue;
        item.amount -= left.amount;
        inv.setItem(i, item);
    }
    dimension.playSound(isBarrel ? 'block.barrel.close' : 'random.chestclosed', block.center());
}
afterEvents.dataDrivenEntityTrigger.subscribe(({ eventId, entity }) => {
    if (eventId !== 'minecraft:from_village' || Math.random() > 0.5)
        return;
    entity.dimension.spawnEntity('rhg:hay_golem', entity.location);
});
afterEvents.playerSpawn.subscribe(({ initialSpawn, player }) => {
    if (!initialSpawn || world.gameRules.mobGriefing)
        return;
    player.sendMessage(`§cThe gamerule "MobGriefing" has been set to false. The Hay Golem will not be able to harvest crops.`);
});
afterEvents.gameRuleChange.subscribe(({ rule, value }) => {
    if (rule !== GameRule.MobGriefing || value)
        return;
    world.sendMessage(`§cThe gamerule "MobGriefing" has been set to false. The Hay Golem will not be able to harvest crops.`);
});
